@@ -20,36 +20,35 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 from libopensesame.py3compat import *
 from qtpy import QtCore, QtWidgets
 from libopensesame.exceptions import osexception
-from libopensesame import debug
+from libopensesame.oslogging import oslogger
 from libqtopensesame.widgets.item_view_button import item_view_button
 from libqtopensesame.widgets.tree_item_item import tree_item_item
 from libqtopensesame.widgets.qtitem_splitter import qtitem_splitter
 from libqtopensesame.widgets import header_widget
 from libqtopensesame._input.pool_select import pool_select
 from libqtopensesame.misc.config import cfg
-from libqtopensesame.misc.base_qtobject import base_qtobject
 from libqtopensesame.misc.translate import translation_context
 _ = translation_context(u'qtitem', category=u'core')
 
 
 def requires_init(fnc):
-	
+
 	"""
 	desc:
 		A decorator that makes sure that an item's controls are initialized
 		before a function is called.
 	"""
-	
+
 	def inner(self, *args, **kwargs):
-		
+
 		if self.container_widget is None:
 			self.init_edit_widget()
 		return fnc(self, *args, **kwargs)
-		
+
 	return inner
 
 
-class qtitem(base_qtobject):
+class qtitem(object):
 
 	"""Base class for the GUI controls of other items"""
 
@@ -78,10 +77,11 @@ class qtitem(base_qtobject):
 			self.container_widget = None
 		else:
 			self.init_edit_widget()
+		self._dirty = False
 		self.lock = False
 		self.maximized = False
 		self.set_validator()
-		debug.msg(u'created %s' % self.name)
+		oslogger.debug(u'created %s' % self.name)
 
 	@property
 	def main_window(self):
@@ -110,7 +110,7 @@ class qtitem(base_qtobject):
 	@property
 	def default_description(self):
 		return _(u'Default description')
-		
+
 	def open_tab(self, select_in_tree=True):
 
 		"""
@@ -122,7 +122,10 @@ class qtitem(base_qtobject):
 		self.tabwidget.add(self.widget(), self.item_icon(), self.name)
 		self.main_window.set_unsaved(unsaved)
 		if select_in_tree:
-			self.experiment.main_window.ui.itemtree.select_item(self.name)
+			self.experiment.main_window.ui.itemtree.select_item(
+				self.name,
+				open_tab=False
+			)
 
 	def close_tab(self):
 
@@ -193,9 +196,11 @@ class qtitem(base_qtobject):
 			Is called when the tab becomes visible, and updated the contents.
 		"""
 
+		self.extension_manager.fire(u'prepare_open_item', name=self.name)
 		self.update_script()
 		self.edit_widget()
 		self.main_window.ui.itemtree.select_item(self.name, open_tab=False)
+		self.extension_manager.fire(u'open_item', name=self.name)
 
 	@requires_init
 	def widget(self):
@@ -296,8 +301,7 @@ class qtitem(base_qtobject):
 		elif self.initial_view == u'split':
 			self.set_view_split()
 		else:
-			debug.msg(u'Invalid initial_view: %s' % self.initial_view,
-				reason=u'warning')
+			oslogger.warning(u'Invalid initial_view: %s' % self.initial_view)
 			self.set_view_controls()
 		self.splitter.splitterMoved.connect(self.splitter_moved)
 		self.container_vbox = QtWidgets.QVBoxLayout()
@@ -377,9 +381,10 @@ class qtitem(base_qtobject):
 		# Items are updated when their tab is shown, so we don't need to update
 		# them if they aren't shown.
 		if self.tabwidget.current_item() != self.name:
-			return
+			return False
 		self.update_script()
 		self.edit_widget()
+		return True
 
 	def update_script(self):
 
@@ -418,6 +423,7 @@ class qtitem(base_qtobject):
 
 		self.auto_apply_edit_changes()
 		self.update_script()
+		self.set_clean()
 		return True
 
 	def apply_script_changes(self, *deprecated, **_deprecated):
@@ -494,7 +500,7 @@ class qtitem(base_qtobject):
 		for cls in inspect.getmro(self.__class__):
 			if meth.__name__ in cls.__dict__:
 				break
-		debug.msg(u'validator: %s' % cls)
+		oslogger.debug(u'validator: %s' % cls)
 		self.validator = cls
 
 	@requires_init
@@ -641,6 +647,9 @@ class qtitem(base_qtobject):
 			type:	bool
 		"""
 
+		if self._dirty:
+			self.apply_edit_changes()
+			return True
 		return False
 
 	def auto_edit_widget(self):
@@ -734,22 +743,19 @@ class qtitem(base_qtobject):
 				if val != qprogedit.text():
 					qprogedit.setText(val)
 
-	def auto_apply_edit_changes(self, rebuild=True):
+	def auto_apply_edit_changes(self):
 
 		"""
 		desc:
 			Applies the auto-widget controls.
-
-		keywords:
-			rebuild:	Deprecated (does nothing).
 		"""
 
 		for var, edit in self.auto_line_edit.items():
 			if isinstance(var, int):
 				continue
 			if edit.isEnabled() and isinstance(var, basestring):
-				val = str(edit.text()).strip()
-				if val != u'':
+				val = edit.text().strip()
+				if val:
 					self.var.set(var, val)
 					continue
 				# If no text was entered, we use a default if available ...
@@ -763,7 +769,7 @@ class qtitem(base_qtobject):
 			if isinstance(var, int):
 				continue
 			if combobox.isEnabled() and isinstance(var, basestring):
-				self.var.set(var, str(combobox.currentText()))
+				self.var.set(var, combobox.currentText())
 
 		for var, spinbox in self.auto_spinbox.items():
 			if isinstance(var, int):
@@ -781,10 +787,7 @@ class qtitem(base_qtobject):
 			if isinstance(var, int):
 				continue
 			if checkbox.isEnabled() and isinstance(var, basestring):
-				if checkbox.isChecked():
-					val = u"yes"
-				else:
-					val = u"no"
+				val = u'yes' if checkbox.isChecked() else u'no'
 				self.var.set(var, val)
 
 		for var, qprogedit in self.auto_editor.items():
@@ -816,29 +819,55 @@ class qtitem(base_qtobject):
 			var = id(widget)
 		if apply_func is None:
 			apply_func = self.apply_edit_changes
-		debug.msg(var)
 		self.set_focus_widget(widget)
-		if isinstance(widget, QtWidgets.QSpinBox) or isinstance(widget,
-			QtWidgets.QDoubleSpinBox):
+		if (
+			isinstance(widget, QtWidgets.QSpinBox) or
+			isinstance(widget, QtWidgets.QDoubleSpinBox)
+		):
 			widget.editingFinished.connect(apply_func)
+			widget.valueChanged.connect(self.set_dirty)
 			self.auto_spinbox[var] = widget
 		elif isinstance(widget, QtWidgets.QComboBox):
 			widget.activated.connect(apply_func)
 			self.auto_combobox[var] = widget
-		elif isinstance(widget, QtWidgets.QSlider) \
-			or isinstance(widget, QtWidgets.QDial):
+		elif (
+			isinstance(widget, QtWidgets.QSlider) or
+			isinstance(widget, QtWidgets.QDial)
+		):
 			widget.valueChanged.connect(apply_func)
 			self.auto_slider[var] = widget
-		elif isinstance(widget, QtWidgets.QLineEdit) or isinstance(widget,
-			pool_select):
+		elif (
+			isinstance(widget, QtWidgets.QLineEdit) or
+			isinstance(widget, pool_select)
+		):
 			widget.editingFinished.connect(apply_func)
+			widget.textEdited.connect(self.set_dirty)
 			self.auto_line_edit[var] = widget
 		elif isinstance(widget, QtWidgets.QCheckBox):
 			widget.clicked.connect(apply_func)
 			self.auto_checkbox[var] = widget
 		else:
-			raise Exception(u"Cannot auto-add widget of type %s" % widget)
-			
+			raise TypeError(u"Cannot auto-add widget of type %s" % widget)
+
+	def set_dirty(self):
+
+		"""
+		desc:
+			Indicates that the item's controls have changed since being last
+			applied.
+		"""
+
+		self._dirty = True
+
+	def set_clean(self):
+
+		"""
+		desc:
+			Indicates that the item's controls have all been applied.
+		"""
+
+		self._dirty = False
+
 	def children(self):
 
 		"""
